AES
	proc
		//Takes a key as a string and returns the expanded key as a matrix
		KeyExpansion(key)
			var/retVal[44][4]
			for(var/i = 1; i <= 4; i++)
				for(var/j = 1; j <= 4; j++)
					retVal[i][j] = key[(i-1)*4 + j]
			for(var/i = 5; i <= 44; i++)
				var/temp = retVal[i-1]
				if((i-1) % 4 == 0)
					temp = XOR(Substitute(Rotate(temp)), RCon((i-1)/4))
				retVal[i] = XOR(retVal[i-4], temp)
			return retVal

		//Takes a list of 4 ints and returns a new list with the first element
		//in the last position. All other elements move up 1 position
		Rotate(list/L)
			var/retVal[4]
			for(var/i = 1; i < 4; i++)
				retVal[i] = L[i+1]
			retVal[4] = L[1]
			return retVal

		//Takes a list of 4 ints and returns a new list according to the
		//substitution table
		Substitute(list/L)
			var/retVal[4]
			for(var/i = 1; i <= 4; i++)
				retVal[i] = Sbox[L[i]+1]
			return retVal

		//Takes an i and returns a Round Constant matrix according to the
		//Rcon table
		RCon(i)
			var/list/retVal = list(Rcon[i+1], 0, 0, 0)
			return retVal

		//Takes two int lists and returns their XOR
		XOR(list/A, list/B)
			var/list/retVal = new/list()
			for(var/i = 1; i <= 4; i++)
				retVal.Add(A[i]^B[i])
			return retVal

		//Takes a matrix of 16 ints and shifts the rows according to the
		//rijndael algorithm
		ShiftRow(list/block)
			var/temp = block[1][2]
			block[1][2]=block[2][2]
			block[2][2]=block[3][2]
			block[3][2]=block[4][2]
			block[4][2]=temp
			temp = block[1][3]
			var/temp2 = block[2][3]
			block[1][3]=block[3][3]
			block[2][3]=block[4][3]
			block[3][3]=temp
			block[4][3]=temp2
			temp = block[4][4]
			block[4][4]=block[3][4]
			block[3][4]=block[2][4]
			block[2][4]=block[1][4]
			block[1][4]=temp

		//Inverse of ShiftRow
		I_ShiftRow(list/block)
			var/temp = block[4][2]
			block[4][2]=block[3][2]
			block[3][2]=block[2][2]
			block[2][2]=block[1][2]
			block[1][2]=temp
			temp = block[4][3]
			var/temp2 = block[3][3]
			block[4][3]=block[2][3]
			block[3][3]=block[1][3]
			block[2][3]=temp
			block[1][3]=temp2
			temp = block[1][4]
			block[1][4]=block[2][4]
			block[2][4]=block[3][4]
			block[3][4]=block[4][4]
			block[4][4]=temp

		//Takes a matrix of 16 ints and mixes the columns according to the
		//rijndael algorithm
		MixColumns(list/block)
			var/retVal[4][4]
			for(var/i = 1; i <= 4; i++)
				for(var/j = 1; j <= 4; j++)
					var/a = ByteMul(block[i][1], Mix[(j-1)*4+1])
					var/b = ByteMul(block[i][2], Mix[(j-1)*4+2])
					var/c = ByteMul(block[i][3], Mix[(j-1)*4+3])
					var/d = ByteMul(block[i][4], Mix[(j-1)*4+4])
					retVal[i][j] = a^b^c^d
			return retVal

		//Inverse of MixColumns
		I_MixColumns(list/block)
			var/retVal[4][4]
			for(var/i = 1; i <= 4; i++)
				for(var/j = 1; j <= 4; j++)
					var/a = ByteMul(block[i][1], I_Mix[(j-1)*4+1])
					var/b = ByteMul(block[i][2], I_Mix[(j-1)*4+2])
					var/c = ByteMul(block[i][3], I_Mix[(j-1)*4+3])
					var/d = ByteMul(block[i][4], I_Mix[(j-1)*4+4])
					retVal[i][j] = a^b^c^d
			return retVal

		//Takes two ints and uses a Galois lookup table to compute
		//their multiplication in Galois 256
		ByteMul(A, B)
			if(B == 2) return G2[A+1]
			else if(B == 3) return G3[A+1]
			else if(B == 9) return G9[A+1]
			else if(B == 11) return G11[A+1]
			else if(B == 13) return G13[A+1]
			else if(B == 14) return G14[A+1]
			else return A

		//XOR's the round key with the block
		AddRoundKey(list/block,list/key, round)
			for(var/i = 1; i <= 4; i++)
				block[i] = XOR(block[i], key[(round-1)*4+i])

		//Substitutes elements of the matrix according
		//to the rijndael algorithm
		SubstituteBlock(list/block)
			for(var/i = 1; i <= 4; i++)
				for(var/j = 1; j<= 4; j++)
					block[i][j] = Sbox[block[i][j]+1]

		//Inverse of SubstituteBlock
		I_SubstituteBlock(list/block)
			for(var/i = 1; i <= 4; i++)
				for(var/j = 1; j <= 4; j++)
					block[i][j] = I_Sbox[block[i][j]+1]

		//Encrypts a 4x4 matrix of ints, given a key vector
		//Returns a 4x4 matrix of ints
		EncBlock(data, key)
			var/t_key = KeyExpansion(key)
			var/t_data = data
			AddRoundKey(t_data, t_key, 1)
			for(var/i = 2; i <= 10; i++)
				SubstituteBlock(t_data)
				ShiftRow(t_data)
				t_data = MixColumns(t_data)
				AddRoundKey(t_data,t_key,i)
			SubstituteBlock(t_data)
			ShiftRow(t_data)
			AddRoundKey(t_data, t_key, 11)
			return t_data

		//Decrypts a 4x4 matrix of ints, given a key vector
		//Returns a 4x4 matrix of ints
		DecBlock(data, key)
			var/t_key = KeyExpansion(key)
			var/t_data = data
			AddRoundKey(t_data, t_key, 11)
			for(var/i = 10; i >= 2; i--)
				I_ShiftRow(t_data)
				I_SubstituteBlock(t_data)
				AddRoundKey(t_data, t_key, i)
				t_data = I_MixColumns(t_data)
			I_ShiftRow(t_data)
			I_SubstituteBlock(t_data)
			AddRoundKey(t_data, t_key, 1)
			return t_data