AES
	proc
		#if INPUT_MODE == 0
		//Arguments
		//		IV - a 16 character initialization vector
		//		data - the data (as text) to encrypt
		//		key - a 16 character key
		//Returns a matrix representation of the encrypted data
		Encrypt(IV, data, key)
			#if defined(ERROR_MODE) && ERROR_MODE > 0
			if(length(key) != 16 || length(IV) != 16)
				#if ERROR_MODE == 1
				return FALSE
				#elif ERROR_MODE == 2
				return new /AES_Error("Invalid Key or IV Length",
					data, key, IV)
				#endif
			#endif
			var/blocks = round(length(data)/16)
			if(length(data)%16) blocks += 1
			var/retVal[4*blocks][4]
			var/t_IV = text2mat(IV)
			var/t_data = text2mat(data)
			var/t_key = text2int(key)
			var/storage[4][4]
			for(var/i = 1; i <= blocks; i++)
				//Copy the block to storage
				for(var/j = 1; j <= 4; j++)
					for(var/k = 1; k <= 4; k++)
						storage[j][k] = t_data[(i-1)*4+j][k]
				//XOR the storage withth the IV
				for(var/j = 1; j <= 4; j++)
					storage[j] = XOR(storage[j], t_IV[j])
				//Encrypt the storage with our key
				storage = EncBlock(storage,t_key)
				//Copy storage back to retVal
				for(var/j = 1; j <= 4; j++)
					for(var/k = 1; k <= 4; k++)
						retVal[(i-1)*4+j][k] = storage[j][k]
				//Set new IV
				for(var/j = 1; j <= 4; j++)
					for(var/k = 1; k <= 4; k++)
						t_IV[j][k] = storage[j][k]
			return retVal

		//Arguments
		//		IV - your 16 character initialization vector
		//		data - the data (as encrypted matrix) to decrypt
		//		key - your 16 character key
		//Returns a text representation of the decrypted data
		Decrypt(IV, list/data, key)
			#if defined(ERROR_MODE) && ERROR_MODE > 0
			if(length(key) != 16 || length(IV) != 16)
				#if ERROR_MODE==1
				return FALSE
				#elif ERROR_MODE==2
				return new /AES_Error("Invalid Key or IV Length",
					data, key, IV)
				#endif
			#endif
			var/blocks = length(data)/4
			var/retVal
			var/t_IV = text2mat(IV)
			var/t_data = data.Copy()
			var/t_key = text2int(key)
			var/storage[4][4]
			for(var/i = 1; i <= blocks; i++)
				//Copy the block to storage
				for(var/j = 1; j <= 4; j++)
					for(var/k = 1; k <= 4; k++)
						storage[j][k] = t_data[(i-1)*4+j][k]
				//Decrypt the storage with our key
				storage = DecBlock(storage, t_key)
				//XOR the storage with the IV
				for(var/j = 1; j <= 4; j++)
					storage[j] = XOR(storage[j], t_IV[j])
				//Copy storage to retVal
				retVal += mat2text(storage)
				//Set new IV
				for(var/j = 1; j <= 4; j++)
					for(var/k = 1; k <= 4; k++)
						t_IV[j][k] = t_data[(i-1)*4+j][k]
			return retVal
		#else
		//Arguments
		//		IV - a 128 bit hex string initialization vector
		//		data - the data (as a hex string) to encrypt
		//		key - a 128 bit hex string key
		//Returns a hex string representation of the encrypted data
		Encrypt(IV, data, key)
			#if defined(ERROR_MODE) && ERROR_MODE > 0
			if(length(key) != 32 || length(IV) != 32)
				#if ERROR_MODE == 1
				return FALSE
				#elif ERROR_MODE == 2
				return new /AES_Error("Invalid Key or IV Length",
					data, key, IV)
				#endif
			#endif
			var/blocks = round(length(data)/32)
			if(length(data)%32) blocks += 1
			var/retVal[4*blocks][4]
			var/t_IV = hex2mat(IV)
			var/t_data = hex2mat(data)
			var/t_key = hex2int(key)
			var/storage[4][4]
			for(var/i = 1; i <= blocks; i++)
				//Copy the block to storage
				for(var/j = 1; j <= 4; j++)
					for(var/k = 1; k <= 4; k++)
						storage[j][k] = t_data[(i-1)*4+j][k]
				//XOR the storage with the IV
				for(var/j = 1; j <= 4; j++)
					storage[j] = XOR(storage[j], t_IV[j])
				//Encrypt the storage with our key
				storage = EncBlock(storage,t_key)
				//Copy storage back to retVal
				for(var/j = 1; j <= 4; j++)
					for(var/k = 1; k <= 4; k++)
						retVal[(i-1)*4+j][k] = storage[j][k]
				//Set new IV
				for(var/j = 1; j <= 4; j++)
					for(var/k = 1; k <= 4; k++)
						t_IV[j][k] = storage[j][k]
			return mat2hex(retVal)

		//Arguments
		//		IV - your 128 bit hexadecimal initialization vector
		//		data - the data (as hex string) to decrypt
		//		key - your 128 bit hexadecimal key
		//Returns a hexadecimal string representation of the decrypted data
		Decrypt(IV, data, key)
			#if defined(ERROR_MODE) && ERROR_MODE > 0
			if(length(key) != 32 || length(IV) !=32)
				#if ERROR_MODE==1
				return FALSE
				#elif ERROR_MODE==2
				return new /AES_Error("Invalid Key or IV Length",
					data, key, IV)
				#endif
			#endif
			var/retVal
			var/t_IV = hex2mat(IV)
			var/t_data = hex2mat(data)
			var/t_key = hex2int(key)
			var/blocks = length(t_data)/4
			var/storage[4][4]
			for(var/i = 1; i <= blocks; i++)
				//Copy the block to storage
				for(var/j = 1; j <= 4; j++)
					for(var/k = 1; k <= 4; k++)
						storage[j][k] = t_data[(i-1)*4+j][k]
				//Decrypt the storage with our key
				storage = DecBlock(storage, t_key)
				//XOR the storage with the IV
				for(var/j = 1; j <= 4; j++)
					storage[j] = XOR(storage[j], t_IV[j])
				//Copy storage to retVal
				retVal += mat2hex(storage)
				//Set new IV
				for(var/j = 1; j <= 4; j++)
					for(var/k = 1; k <= 4; k++)
						t_IV[j][k] = t_data[(i-1)*4+j][k]
			return retVal
		#endif