/*
This file contains the instructions for use for the AES-128 library.
Please follow these instructions or I cannot guarantee the integrity
of your data. SIDENOTE: I do not guarantee the integrity of your data
even if you follow the instructions here. Bugs happen!

----------------------------------------------------------------------
General Info
----------------------------------------------------------------------
This library implements AES-128. It utilizes 128 bit keys to encrypt
blocks of 128 bits in length. To encrypt larger portions of text the
library utilizes the Cipher Block Chaining method of encryption and
decryption.

You are free to use, modify, and redistribute this code as you see fit.

----------------------------------------------------------------------
Prior to Use
----------------------------------------------------------------------
Before using any of the methods below you must create an instance of
the AES datum. An example of one method of this is found here:

var/AES/aes = new()

I recommend keeping this as a global variable, if possible, to prevent
creating a new object every time you want to encrypt and decrypt.

----------------------------------------------------------------------
Input Modes
----------------------------------------------------------------------
There are two input mode settings which may be found in the
_Settings.dm file under the INPUT_MODE definition. Setting INPUT_MODE
to 0 will allow you to encrypt and decrypt data represented by ASCII
characters. Setting INPUT_MODE to 1 will allow you to encrypt and
decrypt data represented by hexadecimal strings. Please see the
encryption and decryption section to see how this affects you. The
default mode is ASCII which is 0.

----------------------------------------------------------------------
Errors
----------------------------------------------------------------------
There are different error reporting settings which may be found in the
_Settings.dm file under the ERROR_MODE definition. Setting ERROR_MODE
to 0 will turn off error reporting and any known errors will result
in the proc crashing. Setting ERROR_MODE to 1 will gracefully exit on
all known errors and return FALSE. Setting ERROR_MODE to 2 will also
gracefully exit on all known errors and return an AES_Error datum The
default value for ERROR_MODE is 1.

The AES_Error datum has four variables.
	reason - this variable provides a textual reason for failure
	data - this variable reports the data that was to be encrypted
	key - the key used for encryption
	IV - the initialization vector used for encryption



----------------------------------------------------------------------
Encryption
----------------------------------------------------------------------
This section assumes you called your instance of the AES datum aes.

To encrypt text call aes.Encrypt(IV, data, key)

If you're using ASCII mode (INPUT_MODE = 0) then
	IV must be a 16 character long text string
	data can be a text string of any length
	key must be a 16 character long text string
This method will return a matrix (aka 2D list, 2D array, etc.). This matrix
should be passed to the decryption method detailed below to retrieve the
original message.

If you're using Hex mode (INPUT_MODE = 1) then
	IV must be a 128 bit long hexadecimal string
	data can be a hexadecimal string of any length
	key must be a 128 bit long hexadecimal string
This method will return a hexadecimal string that should be passed to
the decryption method detailed below to retrieve the original message.

----------------------------------------------------------------------
Decryption
----------------------------------------------------------------------
This section assumes you called your instance of the AES datum aes.

To decrypt a matrix call aes.Decrypt(IV, data, key)

If you're using ASCII mode (INPUT_MODE = 0) then
	IV must be the same 16 character long text string you used to encrypt
	data must be the matrix returned from the encryption method
	key must be the same 16 character long text string you used to encrypt
This method will return the original text that you encrypted.

If you're using Hex mode (INPUT_MODE = 1) then
	IV must be the same 128 bit hexadecimal string used for encryption
	data must be the hexadecimal string returned from the encryption method
	key must be the same 128 bit hexadecimal string used for encryption
This method will return the original hexadecimal string that you encrypted
*/